/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// GenomeBrowser Project
//
// GenomeBrowser.java
// Since: Apr 20, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/GenomeBrowser.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client;

import java.util.HashMap;

import org.utgenome.gwt.utgb.client.db.DatabaseService;
import org.utgenome.gwt.utgb.client.db.DatabaseServiceAsync;
import org.utgenome.gwt.utgb.client.model.TrackSelector;
import org.utgenome.gwt.utgb.client.track.GenomeViewer;
import org.utgenome.gwt.utgb.client.track.TrackFactorySetup;
import org.utgenome.gwt.utgb.client.track.TrackGroup;
import org.utgenome.gwt.utgb.client.track.TrackQueue;
import org.utgenome.gwt.utgb.client.track.lib.old.OldUTGBViewer;
import org.utgenome.gwt.utgb.client.util.BrowserInfo;
import org.utgenome.gwt.web.client.tab.TabViewer;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.RootPanel;

/**
 * {@link GenomeBrowser} is an entry point for the web application.
 * 
 * In order to access the web server through {@link BrowserService} interface, 
 * use the {@link #getService()} method:
 * <p>Example</p>
 * <code>
 * BrowserServiceAsyc service = GenomeBrowser.getService();
 * service.getHTTPContent("http://www.google.com", new AsyncCallback()
 *          public void onFailure(Throwable caught)
 *          {
 *              
 *          }
 *
 *          public void onSuccess(Object result)
 *          {
 *             String content = (String) result;
 *             // do something with the given content
 *          }});
 * ...
 * </code>
 * 
 * @author leo
 *
 */
public class GenomeBrowser implements EntryPoint
{
    private TabViewer tabViewer = new TabViewer();
    
    private static BrowserServiceAsync _service = null;
    private static DatabaseServiceAsync _dbService = null;
 
    private static TrackGroup _rootGroup = new TrackGroup("root");
    private static TrackQueue _mainQueue = new TrackQueue(_rootGroup);
    
 
    
    public static void initServices()
    {
        // set up an access interface to the web service 
        _service = (BrowserServiceAsync) GWT.create(BrowserService.class);
        ((ServiceDefTarget) _service).setServiceEntryPoint(GWT.getModuleBaseURL() + "service");
        
        _dbService = (DatabaseServiceAsync) GWT.create(DatabaseService.class);
        ((ServiceDefTarget) _dbService).setServiceEntryPoint(GWT.getModuleBaseURL() + "database");
    }
     
    public void onModuleLoad()
    {
        initServices();
    	// list up track factories
    	TrackFactorySetup.initialize();    
 
    	
        // dispatch the program according to the URL query 
        HashMap request = BrowserInfo.getURLQueryRequestParameters();
        
        int initialTabIndex = 0;
        tabViewer.addTabEntry(GenomeViewer.factory());
        tabViewer.addTabEntry(TrackSelector.factory());
        tabViewer.addTabEntry(OldUTGBViewer.factory());
        
        tabViewer.selectTab(initialTabIndex);
        
        RootPanel.get().add(tabViewer);
    }
    
    public static BrowserServiceAsync getService()
    {
    	if(_service == null)
    		throw new IllegalStateException("BrowserService is not initialized");
        return _service;
    }

    public static DatabaseServiceAsync getDatabaseService()
    {
    	if(_dbService == null)
    		throw new IllegalStateException("DatabaseService is not initialized");
    	return _dbService;
    }
    
    public static TrackGroup getRootTrackGroup()
    {
    	return _rootGroup;
    }
    public static TrackQueue getTrackQueue()
    {
    	return _mainQueue;
    }
    
     
    public static void hideLoadingMessage()
    {
        Element _loadingMessage = DOM.getElementById("loading");
        if(_loadingMessage != null)
        {
            RootPanel.setVisible(_loadingMessage, false);
        }
    }
    
    public static void showLoadingMessage()
    {
        Element _loadingMessage = DOM.getElementById("loading");
        if(_loadingMessage != null)
        {
            RootPanel.setVisible(_loadingMessage, true);
        }
        
    }
    
}




