/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// GenomeBrowser Project
//
// DatabaseServiceAsync.java
// Since: Jul 2, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/db/DatabaseServiceAsync.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client.db;


import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.RemoteService;

public interface DatabaseServiceAsync extends RemoteService {

	/**
     * Queries database data  
     * @param jsonQuery	the query generted from {@link Query#toJSON}
     * @return the query result 
     * @see {@link Query}
     */
    public void query(String path, String jsonQuery, AsyncCallback callback);
    
    
    // file server data
    
    /**
     * Gets list of the database catalog data hold by this database server
     * @return JSON representation of database catalogs
     */
    public void getCatalog(AsyncCallback callback);
    
    /**
     * Gets the list of database server locations (IP addresses) of the given path 
     * @return the list of database servers (JSON format)
     */
    public void getDatabaseLocation(String path, AsyncCallback callback);

    
    /**
     * Gets the list of available species
     * @return the list of species
     */
    public void getAvailableSpecies(AsyncCallback callback);
    
    /**
     * Gets the list of sequence revisions for the species
     * @return the list of sequence revisions
     */
    public void getSequenceRevision(String species, AsyncCallback callback);
    
    
    /**
     * Gets the list of LegacyTrackInfo
     * @return the list of LegacyTrackInfo
     */
    public void getLegacyTrackList(String species, String sequenceRevision, AsyncCallback callback);

	
}




