/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// UTGB Gallery Project
//
// DataModelGraph.java
// Since: Apr 20, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/model/DataModelGraph.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client.model;

import java.util.ArrayList;
import java.util.Iterator;

import org.utgenome.gwt.utgb.client.BrowserService;
import org.utgenome.gwt.utgb.client.BrowserServiceAsync;
import org.utgenome.gwt.web.client.UTGBClientException;

import com.google.gwt.core.client.GWT;
import com.google.gwt.json.client.JSONArray;
import com.google.gwt.json.client.JSONNumber;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.json.client.JSONString;
import com.google.gwt.json.client.JSONValue;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.VerticalPanel;


class DataModelNode
{
    private int id;
    private String name;
    
    
    
    /**
     * @param id
     * @param name
     */
    public DataModelNode(int id, String name)
    {
        this.id = id;
        this.name = name;
    }
    
    public DataModelNode(JSONObject json) throws UTGBClientException
    {
        if(json == null)
            throw new UTGBClientException("input json data is null");
        
        JSONNumber idData;
        JSONValue v = json.get("id");
        if(v != null && (idData = v.isNumber()) != null)
        {
            this.id = (int) idData.getValue();
        }
        
        JSONString s;
        JSONValue v2 = json.get("name");
        if(v2 != null && (s = v2.isString()) != null)
        {
            this.name = s.stringValue();
        }
    }
    
    public int getId()
    {
        return id;
    }
    public void setId(int id)
    {
        this.id = id;
    }
    public String getName()
    {
        return name;
    }
    public void setName(String name)
    {
        this.name = name;
    }
    
    
}

class DataModelEdge
{
    public final int ONE_TO_ONE = 0;
    public final int ONE_TO_MANY = 1;
    public final int MANY_TO_MANY = 2;
    
    private int from;
    private int to;
    private int relationType;
        
    
    
}

/**
 * The DataModelGraph is a graphical interface to select tracks associated to several
 * parameter values, e.g., a genome sequence revision, species, target  etc.
 *  
 * @author leo
 *
 */
public class DataModelGraph extends Composite
{
    private ArrayList _nodeList = new ArrayList();
    private ArrayList _edgeList = new ArrayList(); 
    
    private DockPanel _panel = new DockPanel();

    BrowserServiceAsync _service;
    private ServiceDefTarget _endPoint;
    private String _moduleRelativeURL;

    public DataModelGraph() 
    {
        // setup asynchronous call
        _service = (BrowserServiceAsync) GWT.create(BrowserService.class);
        _endPoint = (ServiceDefTarget) _service;
        _moduleRelativeURL = GWT.getModuleBaseURL() + "service";
        _endPoint.setServiceEntryPoint(_moduleRelativeURL);

        
        initWidget(_panel);
    }
    
    public void update(JSONValue value) throws UTGBClientException
    {
        JSONObject jsonObj;
        ArrayList nodeList = new ArrayList();
        ArrayList edgeList = new ArrayList();
        if((jsonObj = value.isObject()) != null)
        {
            JSONValue n = jsonObj.get("node");
            JSONArray nodeData;
            if((nodeData = n.isArray()) == null)
                throw new UTGBClientException("json data has no node entry: " + value.toString());

            // add nodes
            for(int i=0; i<nodeData.size(); i++)
                nodeList.add(new DataModelNode(nodeData.get(i).isObject()));

            
            
            JSONArray edgeData;
            JSONValue a = jsonObj.get("edge");
            if((edgeData = a.isArray()) != null)
            {
                
                
            }

        }
        
        
        _nodeList = nodeList;
        _edgeList = edgeList;

        
        _panel.clear();
        
        VerticalPanel v = new VerticalPanel();
        
        for(Iterator it = _nodeList.iterator(); it.hasNext(); )
        {
            DataModelNode node = (DataModelNode) it.next();
            
            HorizontalPanel hp = new HorizontalPanel();
            Label l = new Label(node.getName());
            l.setStyleName("datamodel-label");
            
            hp.add(l);
            v.add(hp);
        }
        _panel.add(v, DockPanel.CENTER);
    }
    
    
}




