/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// GenomeBrowser Project
//
// TrackQueue.java
// Since: Jun 6, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/track/TrackQueue.java $ 
// $Author: leo $s
//--------------------------------------
package org.utgenome.gwt.utgb.client.track;

import java.util.Iterator;

import org.utgenome.gwt.utgb.client.GenomeBrowser;
import org.utgenome.gwt.utgb.client.ui.WindowBox;
import org.utgenome.gwt.utgb.client.ui.WindowListener;
import org.utgenome.gwt.web.client.ui.DraggableTable;

import com.google.gwt.user.client.ui.Composite;

/**
 * {@link TrackQueue} is a widget that draws tracks
 * 
 * @author leo
 * 
 */
public class TrackQueue extends Composite implements TrackUpdateListener, TrackLayoutManager, WindowListener
{
    private TrackGroup _trackGroup;
    private final DraggableTable _trackQueue = new DraggableTable();

    public TrackQueue(TrackGroup trackGroup)
    {
        this._trackGroup = trackGroup;
        init();
        initWidget(_trackQueue);
    }

    private void init()
    {
        _trackQueue.setStyleName("track-queue");
        initTrackGroup(_trackGroup);
    }

    private void initTrackGroup(TrackGroup trackGroup)
    {
        trackGroup.setTrackLayoutManager(this);
        trackGroup.addTrackUpdateListener(this);
    }

    public void onDetachedFromTrackGroup(TrackGroup trackGroup)
    {
        _trackQueue.clear();
        // initTrackGroup(trackGroup);

        // sync(trackGroup);
    }

    private void sync(TrackGroup group)
    {
        // resync TrackQueue and TrackGroup
        for (Iterator it = group.getTrackList().iterator(); it.hasNext();)
        {
            Track track = (Track) it.next();
            onInsertTrack(track);
        }
        for (Iterator it = group.getTrackGroupList().iterator(); it.hasNext();)
        {
            TrackGroup trackGroup = (TrackGroup) it.next();
            sync(trackGroup);
        }

        // remove loading message
        GenomeBrowser.hideLoadingMessage();
    }

    public WindowBox createWindow()
    {
        WindowBox win = new WindowBox(this);
        win.addWindowListener(this);
        return win;
    }

    public void adjustHeight()
    {
        int height = _trackGroup.getHeight();
        _trackQueue.setHeight(height + "px");
    }

    public int getTrackIndex(Track track)
    {
        return _trackQueue.getIndex(track.getFrame());
    }

    public void onRemoveTrack(Track track)
    {
        _trackQueue.remove(track.getFrame());

        /*
         * // when track queue is empty if(_trackQueue.empty()) {
         * _trackGroup.addTrack(new ToolBoxTrack()); }
         */
    }

    public void onInsertTrack(Track track)
    {
        onInsertTrack(track, -1);
    }

    public void onInsertTrack(Track track, int beforeIndex)
    {
        TrackFrame frame = new TrackFrame(track, _trackGroup.getTrackWindow().getWindowWidth(), track
                .getDefaultWindowHeight());

        if (beforeIndex < 0)
            _trackQueue.add(frame, frame.getFrameBar()); // insert to the
        // tail of the queue
        else
            _trackQueue.insert(frame, frame.getFrameBar(), beforeIndex);

        // draw the track
        track.refresh();
    }

    public void onAddTrackGroup(TrackGroup trackGroup)
    {
        initTrackGroup(trackGroup);
        for (Iterator it = trackGroup.getTrackList().iterator(); it.hasNext();)
        {
            Track track = (Track) it.next();
            onInsertTrack(track);
        }
        for (Iterator it = trackGroup.getTrackGroupList().iterator(); it.hasNext();)
        {
            TrackGroup group = (TrackGroup) it.next();
            onAddTrackGroup(group);
        }

    }

    public void onRemoveTrackGroup(TrackGroup trackGroup)
    {
        for (Iterator it = trackGroup.getTrackList().iterator(); it.hasNext();)
        {
            Track track = (Track) it.next();
            onRemoveTrack(track);
        }
        for (Iterator it = trackGroup.getTrackGroupList().iterator(); it.hasNext();)
        {
            TrackGroup group = (TrackGroup) it.next();
            onRemoveTrackGroup(group);
        }
    }

    public void onResizeTrack()
    {
        adjustHeight();
    }

    protected void onLoad()
    {
        for (Iterator it = _trackGroup.getAllTrackList().iterator(); it.hasNext();)
        {
            Track track = (Track) it.next();
            track.getFrame().adjustFrameHeight();
        }
        adjustHeight();
    }

    public void onButtonClick(WindowBox window, int buttonType)
    {
    // TODO Auto-generated method stub

    }

    public void onResizeWindow(WindowBox window, int newWindowSize)
    {
        int trackWindowWidth = newWindowSize - TrackFrame.INFOPANEL_WIDTH;
        if (trackWindowWidth <= 0)
            trackWindowWidth = 1;

        TrackWindow trackWindow = _trackGroup.getTrackWindow();
        TrackGroupPropertyWriter writer = _trackGroup.getPropertyWriter();
        writer.setTrackWindowSize(trackWindowWidth);
    }

    public void onResizeTrackWindow(int newWindowSize)
    {
        _trackQueue.setWidth((TrackFrame.INFOPANEL_WIDTH + newWindowSize + TrackFrame.SCROLLBAR_WIDTH) + "px");
    }

}
