/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// GenomeBrowser Project
//
// TrackViewer.java
// Since: Jun 5, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/track/TrackViewer.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client.track;


import org.utgenome.gwt.utgb.client.track.lib.LegacyTrack;
import org.utgenome.gwt.utgb.client.track.lib.LegacyTrackInfo;
import org.utgenome.gwt.utgb.client.track.lib.OperationTrack;
import org.utgenome.gwt.utgb.client.track.lib.RevisionSelectTrack;
import org.utgenome.gwt.utgb.client.track.lib.RulerTrack;
import org.utgenome.gwt.utgb.client.track.lib.ScrollButtonTrack;
import org.utgenome.gwt.utgb.client.track.lib.SequenceRulerTrack;
import org.utgenome.gwt.utgb.client.track.lib.SpeciesSelectTrack;
import org.utgenome.gwt.utgb.client.track.lib.ToolBoxTrack;
import org.utgenome.gwt.utgb.client.track.lib.debug.PropertyEditTrack;
import org.utgenome.gwt.utgb.client.track.lib.debug.SQLiteTrack;
import org.utgenome.gwt.utgb.client.track.lib.debug.SampleGeneTrackGroup;

import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.DockPanel;

public class TrackViewer extends Composite 
{
    DockPanel _panel = new DockPanel();
    
    private TrackGroup _globalTrackGroup = new TrackGroup("Viewer"); 
    private TrackQueue _trackQueue = new TrackQueue(_globalTrackGroup);
    
    public TrackViewer()
    {
        _globalTrackGroup.getPropertyWriter().setTrackWindowSize(800);
        _globalTrackGroup.addTrack(new ScrollButtonTrack());
        _globalTrackGroup.addTrack(new ToolBoxTrack());
        _globalTrackGroup.addTrack(new SQLiteTrack());
        _globalTrackGroup.addTrack(new PropertyEditTrack());
        _globalTrackGroup.addTrack(new SpeciesSelectTrack());
        _globalTrackGroup.addTrack(new RevisionSelectTrack());
        _globalTrackGroup.addTrack(new SequenceRulerTrack());
        _globalTrackGroup.addTrack(new RulerTrack());
        
        
        TrackGroup sampleGeneTrackGroup = new SampleGeneTrackGroup();
        _globalTrackGroup.addTrackGroup(sampleGeneTrackGroup);
        
        String browserBaseURL = "http://dev.utgenome.org/~utgb/human/genome_browser/tracks/";
        LegacyTrackInfo refseqinfo = new LegacyTrackInfo("human", "hg18", "Refseq Genes", "chr1", browserBaseURL + "easytrack/easygenetrack.php", 0, 10000, browserBaseURL + "easytrack/easygenetrack_index.php");
        LegacyTrack legacyTrack = new LegacyTrack(refseqinfo);
        _globalTrackGroup.addTrack(legacyTrack);

        if ( true ) {
            final String descriptionXMLURL = "http://medaka3.gi.k.u-tokyo.ac.jp/~ssksn/fosmid_end_track/fosmid_end_description.xml";
            final OperationTrack _opTrack = new OperationTrack();
            _opTrack.setStartIndex(800000);
            _opTrack.setBuild("200506");
            _opTrack.setDescriptionXML(descriptionXMLURL);
            _globalTrackGroup.addTrack(_opTrack);
        }
        //final TrackGroup oldUTGBTrackGroup = new OldUTGBTrackGroup();
        //_globalTrackGroup.addTrackGroup(oldUTGBTrackGroup);

        _globalTrackGroup.getPropertyWriter().setTrackWindow(0, 10000);
        _globalTrackGroup.getPropertyWriter().setProperty(UTGBProperty.SEQUENCE_SIZE, "100000000");
        
        _panel.add(_trackQueue, DockPanel.CENTER);
        

        initWidget(_panel);
    }

    
}

