/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// utgb-core Project
//
// NavigatorTrack.java
// Since: Oct 2, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/track/lib/NavigatorTrack.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client.track.lib;

import java.util.Iterator;
import java.util.List;

import org.utgenome.gwt.utgb.client.GenomeBrowser;
import org.utgenome.gwt.utgb.client.track.Track;
import org.utgenome.gwt.utgb.client.track.TrackBase;
import org.utgenome.gwt.utgb.client.track.TrackFrame;
import org.utgenome.gwt.utgb.client.track.TrackGroup;
import org.utgenome.gwt.utgb.client.track.TrackGroupPropertyChange;
import org.utgenome.gwt.utgb.client.track.TrackWindow;
import org.utgenome.gwt.utgb.client.track.UTGBProperty;
import org.utgenome.gwt.utgb.client.ui.FormLabel;
import org.utgenome.gwt.utgb.client.util.xml.XMLWriter;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.Hidden;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.KeyboardListenerAdapter;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * Navigator of the UTGB
 * 
 * @author leo
 * 
 */
public class NavigatorTrack extends TrackBase
{

    public static TrackFactory factory()
    {
        return new TrackFactory() {
            public Track newInstance()
            {
                return new NavigatorTrack();
            }
        };
    }

    public VerticalPanel panel = new VerticalPanel();

    private ListBox speciesBox = new ListBox();
    private ListBox revisionBox = new ListBox();
    private TextBox targetBox = new TextBox();
    private TextBox startBox = new TextBox();
    private TextBox endBox = new TextBox();

    private class PropertyChangeListener implements ChangeListener
    {
        private String proeprty;
        private ListBox listBox;

        public PropertyChangeListener(String property, ListBox listBox)
        {
            this.proeprty = property;
            this.listBox = listBox;
        }

        public void onChange(Widget sender)
        {
            getTrackGroup().getPropertyWriter().setProperty(proeprty, listBox.getItemText(listBox.getSelectedIndex()));
        }
    }

    private class SequenceRangeChangeListner extends KeyboardListenerAdapter
    {
        public void onKeyUp(Widget sender, char keyCode, int modifiers)
        {
            if (keyCode == KeyboardListener.KEY_ENTER || keyCode == KeyboardListener.KEY_TAB)
            {
                try
                {
                    getTrackGroup().setTrackWindowLocation(Integer.parseInt(startBox.getText()),
                            Integer.parseInt(endBox.getText()));
                }
                catch (NumberFormatException e)
                {
                    GWT.log("(" + startBox.getText() + ", " + endBox.getText() + ") is invalid range", e);
                }
            }
        }
    }

    class ScrollButtonSet extends Composite
    {
        HorizontalPanel _panel = new HorizontalPanel();

        class WindowScrollButton extends Button implements ClickListener
        {
            int movePercentageOnWindow;

            public WindowScrollButton(String label, int movePercentageOnWindow)
            {
                super(label);
                setStyleName("scrollbutton");
                addClickListener(this);
                this.movePercentageOnWindow = movePercentageOnWindow;
            }

            public void onClick(Widget sender)
            {
                TrackWindow window = getTrackGroup().getTrackWindow();
                int genomeRange = window.getEndOnGenome() - window.getStartOnGenome();
                if (genomeRange < 0)
                    genomeRange = -genomeRange;

                int offset = (int) (genomeRange * ((double) movePercentageOnWindow / 100.0));
                getTrackGroup().getPropertyWriter().setTrackWindow(window.getStartOnGenome() + offset,
                        window.getEndOnGenome() + offset);
            }
        }

        class ZoomButton extends Button implements ClickListener
        {
            int windowSize;

            public ZoomButton(String label, int windowSize)
            {
                super(label);
                this.windowSize = windowSize;
                setStyleName("scrollbutton");
                addClickListener(this);
            }

            public void onClick(Widget sender)
            {
                TrackWindow currentWindow = getTrackGroup().getTrackWindow();
                int start = currentWindow.getStartOnGenome();
                getTrackGroup().setTrackWindowLocation(start, start + windowSize - 1);
            }
        }

        public ScrollButtonSet()
        {
            _panel.setVerticalAlignment(HorizontalPanel.ALIGN_MIDDLE);

            _panel.add(new WindowScrollButton("<<<< ", -95));
            _panel.add(new WindowScrollButton("<<< ", -50));
            _panel.add(new WindowScrollButton("<< ", -25));
            _panel.add(new WindowScrollButton("< ", -10));
            _panel.add(new WindowScrollButton("> ", 10));
            _panel.add(new WindowScrollButton(">> ", 25));
            _panel.add(new WindowScrollButton(">>> ", 50));
            _panel.add(new WindowScrollButton(">>>> ", 95));

            _panel.add(new FormLabel("Window Size:"));
            _panel.add(new ZoomButton("1K", 1000));
            _panel.add(new ZoomButton("10K", 10000));
            _panel.add(new ZoomButton("100K", 100000));
            _panel.add(new ZoomButton("1M", 1000000));
            _panel.add(new ZoomButton("10M", 10000000));
            _panel.add(new ZoomButton("100M", 100000000));
            initWidget(_panel);
        }
    }

    private final HorizontalPanel hp = new HorizontalPanel();
    private final Track _self = this;

    public NavigatorTrack()
    {
        super("UTGB Navigator");

        panel.setStyleName("toolbox");
        panel.setWidth("100%");

        // speciesBox.addItem("medaka");
        // speciesBox.addItem("medaka-HNI");
        // speciesBox.setSelectedIndex(0);
        retrieveSpeciesList();

        targetBox.setText("scaffold1");

        speciesBox.addChangeListener(new PropertyChangeListener(UTGBProperty.SPECIES, speciesBox));
        revisionBox.addChangeListener(new PropertyChangeListener(UTGBProperty.REVISION, revisionBox));
        startBox.addKeyboardListener(new SequenceRangeChangeListner());
        endBox.addKeyboardListener(new SequenceRangeChangeListner());
        targetBox.addKeyboardListener(new KeyboardListenerAdapter() {
            public void onKeyUp(Widget sender, char keyCode, int modifiers)
            {
                if (keyCode == KeyboardListener.KEY_ENTER || keyCode == KeyboardListener.KEY_TAB)
                {
                    getTrackGroup().getPropertyWriter().setProperty(UTGBProperty.TARGET, targetBox.getText());
                }
            }
        });

        targetBox.setWidth("100px");

        // value selectors

        hp.setVerticalAlignment(HorizontalPanel.ALIGN_MIDDLE);
        hp.add(new FormLabel("Species"));
        hp.add(speciesBox);
        hp.add(new FormLabel("Revision"));
        hp.add(revisionBox);
        hp.add(new FormLabel("Target"));
        hp.add(targetBox);

        // window locator
        startBox.setWidth("80px");
        endBox.setWidth("80px");
        HorizontalPanel hp2 = new HorizontalPanel();
        hp2.setVerticalAlignment(HorizontalPanel.ALIGN_MIDDLE);
        hp2.add(new FormLabel("Start"));
        hp2.add(startBox);
        hp2.add(new FormLabel("End"));
        hp2.add(endBox);
        hp2.add(new ScrollButtonSet());

        // save view
        final FormPanel saveViewForm = new FormPanel();
        saveViewForm.setAction(GWT.getModuleBaseURL() + "echo?action=saveView");
        saveViewForm.setEncoding(FormPanel.ENCODING_URLENCODED);
        saveViewForm.setMethod(FormPanel.METHOD_POST);
        final Hidden viewData = new Hidden("view");
        final Button saveButton = new Button("save view");
        HorizontalPanel formLayout = new HorizontalPanel();
        formLayout.setVerticalAlignment(HorizontalPanel.ALIGN_MIDDLE);
        viewData.setVisible(false);
        formLayout.add(viewData);
        formLayout.add(saveButton);
        saveButton.addClickListener(new ClickListener() {
            public void onClick(Widget sender)
            {
                XMLWriter xmlWriter = new XMLWriter();
                getTrackGroup().toXML(xmlWriter);
                String view = xmlWriter.toString();
                viewData.setValue(view);
                saveViewForm.submit();
            }
        });
        saveViewForm.add(formLayout);
        DOM.setStyleAttribute(saveViewForm.getElement(), "margin", "0");

        hp.add(saveViewForm);

        Button loadButton = new Button("load view");
        loadButton.addClickListener(new ClickListener() {
            public void onClick(Widget sender)
            {
                getTrackGroup().insertTrack(new ViewLoaderTrack(), getTrackGroup().getTrackIndex(_self) + 1);
            }
        });
        hp.add(loadButton);

        // layout widgets
        panel.add(hp);
        panel.add(hp2);

    }

    public Widget getWidget()
    {
        return panel;
    }

    private void retrieveSpeciesList()
    {
        GenomeBrowser.getDatabaseService().getAvailableSpecies(new AsyncCallback() {
            public void onFailure(Throwable caught)
            {
                GWT.log("failed to retrieve species list", caught);
            }

            public void onSuccess(Object result)
            {
                List speciesList = (List) result;
                if (speciesList.isEmpty())
                    return;
                speciesBox.clear();
                for (Iterator it = speciesList.iterator(); it.hasNext();)
                {
                    String species = it.next().toString();
                    speciesBox.addItem(species);
                }
                updateListBox();
            }
        });
    }

    private void updateListBox()
    {
        String species = getSelectedSpecies();
        revisionBox.clear();

        GenomeBrowser.getDatabaseService().getSequenceRevision(species, new AsyncCallback() {

            public void onFailure(Throwable caught)
            {
                GWT.log("failed to retrieves sequence revision list", caught);
            }

            public void onSuccess(Object result)
            {
                List revisionList = (List) result;
                if (revisionList.isEmpty())
                    return;
                revisionBox.clear();
                for (Iterator it = revisionList.iterator(); it.hasNext();)
                {
                    String revision = it.next().toString();
                    revisionBox.addItem(revision);
                }

                boolean canSelectRevision = selectItem(revisionBox, getTrackGroup().getPropertyReader().getProperty(
                        UTGBProperty.REVISION));
                if (!canSelectRevision)
                {
                    getTrackGroup().getPropertyWriter().setProperty(UTGBProperty.REVISION, revisionBox.getItemText(0));
                }
            }

        });

    }

    private String getSelectedSpecies()
    {
        return speciesBox.getItemText(speciesBox.getSelectedIndex());
    }

    private boolean selectItem(ListBox listBox, String value)
    {
        for (int i = 0; i < listBox.getItemCount(); i++)
        {
            String itemText = listBox.getItemText(i);
            if (itemText.equals(value))
            {
                listBox.setSelectedIndex(i);
                return true;
            }
        }
        return false;
    }

    public void onChangeTrackGroupProperty(TrackGroupPropertyChange change)
    {
        final String[] relatedProperties = new String[] { UTGBProperty.SPECIES, UTGBProperty.REVISION,
                UTGBProperty.TARGET };
        if (change.containsOneOf(relatedProperties))
        {
            String newSpecies = change.getProperty(UTGBProperty.SPECIES);
            if (newSpecies != null && !newSpecies.equals(getSelectedSpecies()))
            {
                selectItem(speciesBox, newSpecies);
            }
            updateListBox();

            if (change.contains(UTGBProperty.TARGET))
                targetBox.setText(change.getProperty(UTGBProperty.TARGET));

        }
    }

    public void onChangeTrackWindow(TrackWindow newWindow)
    {
        startBox.setText(Integer.toString(newWindow.getStartOnGenome()));
        endBox.setText(Integer.toString(newWindow.getEndOnGenome()));

    }

    public void setUp(TrackFrame trackFrame, TrackGroup group)
    {
        trackFrame.disableClose();

        TrackWindow w = group.getTrackWindow();
        startBox.setText(Integer.toString(w.getStartOnGenome()));
        endBox.setText(Integer.toString(w.getEndOnGenome()));
    }

}
