/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// UTGBMedaka Project
//
// ViewLoaderTrack.java
// Since: Aug 13, 2007
//
// $URL$ 
// $Author$
//--------------------------------------
package org.utgenome.gwt.utgb.client.track.lib;

import org.utgenome.gwt.utgb.client.GenomeBrowser;
import org.utgenome.gwt.utgb.client.track.Track;
import org.utgenome.gwt.utgb.client.track.TrackBase;
import org.utgenome.gwt.utgb.client.track.TrackGroup;
import org.utgenome.gwt.utgb.client.track.TrackLoader;
import org.utgenome.gwt.utgb.client.ui.FormLabel;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FileUpload;
import com.google.gwt.user.client.ui.FormHandler;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.FormSubmitCompleteEvent;
import com.google.gwt.user.client.ui.FormSubmitEvent;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.KeyboardListenerAdapter;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

public class ViewLoaderTrack extends TrackBase {

	public static TrackFactory factory() {
		return new TrackFactory() {
			public Track newInstance() {
				return new ViewLoaderTrack();
			}
		};
	}
	
	VerticalPanel panel = new VerticalPanel();
	TextBox urlBox = new TextBox();
	
	public ViewLoaderTrack() {
		super("View Loader");
		
		// load view via HTTP
		HorizontalPanel hp = new HorizontalPanel();
		hp.setVerticalAlignment(HorizontalPanel.ALIGN_MIDDLE);
		hp.add(new FormLabel("View XML URL: "));
		urlBox.setWidth("400px");
		
		
		urlBox.addKeyboardListener(new KeyboardListenerAdapter(){
			public void onKeyPress(Widget sender, char keyCode, int modifiers) {
				if(keyCode == KeyboardListener.KEY_ENTER)
				{
					downloadView(urlBox.getText());
				}
			}
		});
		Button loadButton = new Button("load");
		loadButton.addClickListener(new ClickListener(){
			public void onClick(Widget sender) {
				downloadView(urlBox.getText());
			}
		});
		
		hp.add(urlBox);
		hp.add(loadButton);
		
		// load view from a file
		final FormPanel fileUploadForm = new FormPanel();
		fileUploadForm.setAction(GWT.getModuleBaseURL() + "echo?action=upload");
		fileUploadForm.setEncoding(FormPanel.ENCODING_MULTIPART);
		fileUploadForm.setMethod(FormPanel.METHOD_POST);
		HorizontalPanel formButtonPanel = new HorizontalPanel();
		FileUpload fileBox = new FileUpload();
		fileBox.setName("file");
		fileBox.setWidth("300px");
		
		Button uploadButton = new Button("submit");
		uploadButton.addClickListener(new ClickListener() {
		    public void onClick(Widget sender)
		    {
		        fileUploadForm.submit();
		    }
		});
		
		formButtonPanel.add(new FormLabel("View XML File:"));
		formButtonPanel.add(fileBox);
		formButtonPanel.add(uploadButton);
		fileUploadForm.add(formButtonPanel);
		DOM.setStyleAttribute(fileUploadForm.getElement(), "margin", "0");
		
		fileUploadForm.addFormHandler(new FormHandler() {
            public void onSubmit(FormSubmitEvent event)
            {
                // do nothing
            }
            public void onSubmitComplete(FormSubmitCompleteEvent event)
            {
                getFrame().setNowLoading();
                GenomeBrowser.getService().getCurrentView(new AsyncCallback()
                {
                    public void onFailure(Throwable caught)
                    {
                        GWT.log("failed to load view", caught);
                    }

                    public void onSuccess(Object result)
                    {
                        String viewXML =  (String) result;
                        setViewXML(viewXML);
                    }
                }
                );
            }
		});
		
	
		// set panes
		panel.add(hp);
		panel.add(fileUploadForm);
	}

	private void downloadView(final String url) 
	{
	    getFrame().setNowLoading();
		GenomeBrowser.getService().getHTTPContent(url, new AsyncCallback(){
			public void onFailure(Throwable caught) {
				GWT.log("failed to load " + url, caught);
				getFrame().loadingDone();
			}
			public void onSuccess(Object result) {
				String viewXML = (String) result;
				setViewXML(viewXML);
                getFrame().loadingDone();
			}
		});
	}
	
	private void setViewXML(String viewXML)
	{
        if(viewXML == null)
            return;
        
        GenomeBrowser.showLoadingMessage();
        GenomeBrowser.getRootTrackGroup().loadFromXML(viewXML);
	}
	
	public Widget getWidget() {
		return panel;
	}

}




