/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// utgb-core Project
//
// TrackFinderTrack.java
// Since: Oct 2, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/client/track/lib/center/TrackFinderTrack.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.client.track.lib.center;

import java.util.Iterator;
import java.util.List;

import org.utgenome.gwt.utgb.client.GenomeBrowser;
import org.utgenome.gwt.utgb.client.track.Track;
import org.utgenome.gwt.utgb.client.track.TrackBase;
import org.utgenome.gwt.utgb.client.track.TrackFrame;
import org.utgenome.gwt.utgb.client.track.TrackGroup;
import org.utgenome.gwt.utgb.client.track.TrackGroupPropertyChange;
import org.utgenome.gwt.utgb.client.track.UTGBProperty;
import org.utgenome.gwt.utgb.client.track.bean.LegacyTrackInfo;
import org.utgenome.gwt.utgb.client.track.lib.old.OldUTGBTrack;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Hyperlink;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * Track resource finder 
 * @author leo
 *
 */
public class TrackFinderTrack extends TrackBase
{
    private final DockPanel dockPanel = new DockPanel();
    private final VerticalPanel listPanel = new VerticalPanel();
    
    public static TrackFactory factory()
    {
        return new TrackFactory() {

            public Track newInstance()
            {
                return new TrackFinderTrack();
            }
        };
    }
    
    
    public TrackFinderTrack()
    {
        super("Track Finder");
        
        dockPanel.add(listPanel, DockPanel.CENTER);
    }

    public Widget getWidget()
    {
        return dockPanel;
    }
    
    

    public int getDefaultWindowHeight()
    {
        return 50;
    }


    public void onChangeTrackGroupProperty(TrackGroupPropertyChange change)
    {
        final String[] relatedProperties = new String[] { UTGBProperty.SPECIES, UTGBProperty.REVISION };
        if(change.containsOneOf(relatedProperties))
        {
            loadTrackDescriptionXMLList();
        }
    }

    protected void loadTrackDescriptionXMLList()
    {
        String species = getTrackGroup().getPropertyReader().getProperty(UTGBProperty.SPECIES);
        String revision = getTrackGroup().getPropertyReader().getProperty(UTGBProperty.REVISION);
        
        if(species == null || revision == null)
            return;
        
        GenomeBrowser.getDatabaseService().getLegacyTrackList(species, revision, new AsyncCallback() {

            public void onFailure(Throwable caught)
            {
                GWT.log("error on loading track descriptions", caught);
            }

            public void onSuccess(Object result)
            {
                listPanel.clear();
                
                List legacyTrackList = (List) result;
                for(Iterator it = legacyTrackList.iterator(); it.hasNext();)
                {
                    LegacyTrackInfo legacyTrackInfo = (LegacyTrackInfo) it.next();
                    listPanel.add(createTrackEntryWidget(legacyTrackInfo));
                }
                
            }
        });
        
    }
    
    private class TrackListClickListener implements ClickListener
    {
        private final String descriptionXML;
        
        public TrackListClickListener(final String descriptionXML)
        {
            this.descriptionXML = descriptionXML;
        }
        
        public void onClick(Widget sender)
        {
            OldUTGBTrack newTrack = new OldUTGBTrack();
            newTrack.setDescriptionXML(descriptionXML);
            getTrackGroup().addTrack(newTrack);
        }
    }
    
    public Widget createTrackEntryWidget(final LegacyTrackInfo trackInfo)
    {
        DockPanel panel = new DockPanel();
        panel.setWidth("100%");

        HorizontalPanel hp = new HorizontalPanel();
        Hyperlink trackNameLabel = new Hyperlink(trackInfo.getName(), "add");
        
        trackNameLabel.addClickListener(new TrackListClickListener(trackInfo.getDescriptionXML()));

        hp.setVerticalAlignment(HorizontalPanel.ALIGN_MIDDLE);
        hp.add(new Image("image/item.gif"));
        hp.add(trackNameLabel);
        hp.add(new HTML(" (<a target=\"_blank\" href=\"" + trackInfo.getDescriptionXML() + "\">" + "descriptionXML</a>)"));
        
        VerticalPanel vp = new VerticalPanel();
        vp.setWidth("100%");
        vp.add(hp);
        
        //HTML description = new HTML(trackInfo.getDescription());
        //description.setStyleName("track-description");
        //vp.add(description);
        
        panel.add(vp, DockPanel.CENTER);
        return panel;
    }


    public void setUp(TrackFrame trackFrame, TrackGroup group)
    {
        loadTrackDescriptionXMLList();
    }
    
    

}




