/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// GenomeBrowser Project
//
// BrowserServiceImpl.java
// Since: Apr 20, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/utgb/server/BrowserServiceImpl.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.utgb.server;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.utgenome.UTGBException;
import org.utgenome.gwt.utgb.client.BrowserService;
import org.utgenome.gwt.utgb.client.track.bean.Result;
import org.utgenome.gwt.utgb.client.track.bean.SearchResult;
import org.utgenome.gwt.utgb.client.track.bean.Tag;
import org.utgenome.gwt.utgb.client.track.bean.TrackBean;
import org.utgenome.gwt.utgb.server.util.WebApplicationResource;
import org.utgenome.gwt.web.client.db.amoeba.Condition;
import org.xerial.core.XerialException;
import org.xerial.db.DBException;
import org.xerial.db.sql.ConnectionPool;
import org.xerial.db.sql.DatabaseAccess;
import org.xerial.db.sql.SQLExpression;
import org.xerial.db.sql.SQLUtil;
import org.xerial.db.sql.sqlite.SQLiteAccess;
import org.xerial.db.sql.sqlite.SQLiteCatalog;
import org.xerial.json.JSONArray;
import org.xerial.json.JSONException;
import org.xerial.json.JSONObject;
import org.xerial.json.JSONValue;
import org.xerial.util.CollectionUtil;
import org.xerial.util.Functor;
import org.xerial.util.StringUtil;
import org.xerial.util.bean.BeanException;
import org.xerial.util.bean.BeanUtil;
import org.xerial.util.log.Logger;


import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * 
 * {"species":{"type":0, "target":"human"}, "genome":{"type":0},
 * "chromosome":{"type":0}}
 * 
 * 
 * @author leo
 * 
 */
public class BrowserServiceImpl extends RemoteServiceServlet implements
		BrowserService {
	

    static Logger _logger = Logger.getLogger(BrowserServiceImpl.class);
	
    /*
    protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		
		PrintWriter writer = response.getWriter();
		String action = request.getParameter("action");
		if(action != null)
		{
			if(action.equals("downloadView"))
			{
				HttpSession session = request.getSession();
				String xml = (String) session.getAttribute("view");
				if(xml != null)
				{
					response.setContentType("application/octet-stream");
					response.addHeader("Content-disposition", "attachment; filename=\"view.xml\"");
					writer.append(xml);
					writer.flush();
				}
			}
			else if(action.equals("getView"))
			{
                HttpSession session = request.getSession();
                _logger.debug("session ID=" + session.getId());
                String xml = (String) session.getAttribute("view");
                if(xml != null)
                {
                    response.setContentType("text/xml");
                    writer.append(xml);
                    writer.flush();
                }
			}
			return;
		}
		writer.append("hello!");
	}
	*/

	
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	SQLiteAccess _query;

	DatabaseAccess _dbAccess;

	ConnectionPool _connectionPool;



	public BrowserServiceImpl() throws DBException {
		/*
		 * _connectionPool = new ConnectionPoolImpl(SQLite.driverName,
		 * SQLite.getDatabaseAddress("mock/tracklist.db")); _dbAccess = new
		 * DatabaseAccess(_connectionPool); _query = new
		 * SQLiteAccess(_dbAccess);
		 */
	}

	public String getDataModelParameter(String amoebaQuery) {
		return null;
	}

	// @see
	// org.utgenome.gwt.utgb.client.BrowserService#countDataModelElementValues(java.lang.String)
	public String countDataModelElementValues(String amoebaQuery) {
		try {
			String sql = amoebaQueryToSQL(amoebaQuery, true);
			List<JSONObject> result = _query.jsonQuery(sql);
			if (result.size() > 0)
				return result.get(0).toJSONString();
		} catch (DBException e) {
			_logger.debug(e);

		}

		// empty result
		return "{}";
	}

	private String getConditionOperator(int conditionType) {
		switch (conditionType) {
		case Condition.GREATER_THAN:
			return ">";
		case Condition.GREATER_THAN_OR_EQUAL:
			return ">=";
		case Condition.LESS_THAN:
			return "<";
		case Condition.LESS_THAN_OR_EQUAL:
			return "<=";
		case Condition.EQUAL:
		default:
			return "=";
		}
	}

	class AmoebaQueryParseResult {
		private Vector<String> paramList = new Vector<String>();

		private HashMap<String, Condition> condition = new HashMap<String, Condition>();

		public AmoebaQueryParseResult(String amoebaQuery) {
			parse(amoebaQuery);
		}

		public boolean hasCondition(String param) {
			return condition.containsKey(param);
		}

		public Condition getCondition(String param) {
			return condition.get(param);
		}

		public Vector<String> getParamList() {
			return paramList;
		}

		/**
		 * @param param
		 * @return sql expression of the condition, or null if no condition is
		 * @throws DBException
		 */
		public String getConditionClause(String param) throws DBException {
			Condition cond = condition.get(param);
			if (cond.isAny())
				return null;
			return SQLExpression.fillTemplate("$1 $2 $3", param,
					getConditionOperator(cond.getType()), SQLUtil
							.doubleQuote(cond.getTarget()));
		}

		public Vector<String> getConditionClause() throws DBException {
			Vector<String> conditionClause = new Vector<String>();
			for (String param : condition.keySet()) {
				String condition = getConditionClause(param);
				if (condition != null)
					conditionClause.add(condition);
			}
			return conditionClause;
		}

		private void parse(String amoebaQuery) {
			JSONObject json;
			try {
				json = new JSONObject(amoebaQuery);
			} catch (JSONException e1) {
				_logger.error(e1);
				return;
			}

			for (String param : json.keys()) {
				paramList.add(param);
				try {
					JSONObject paramCond = json.getJSONObject(param);
					Condition condition = new Condition();
					BeanUtil.populateBean(condition, paramCond);
				} catch (JSONException e) {
					_logger.error(e);
				} catch (BeanException e) {
					_logger.error(e);
				}
			}

		}
	}

	public String amoebaQueryToSQL(String amoebaQuery, boolean isCountQuery)
			throws DBException {
		AmoebaQueryParseResult query = new AmoebaQueryParseResult(amoebaQuery);

		String projection;
		if (isCountQuery)
			projection = StringUtil.join(CollectionUtil.collect(query
					.getParamList(), new CountQuery()), ", ");
		else
			projection = StringUtil.join(query.getParamList(), ", ");

		String sql = SQLExpression.fillTemplate("select $1 from $2 $3",
				projection, "entry",
				(query.getConditionClause().size() > 0) ? "where "
						+ StringUtil.join(query.getConditionClause(), " and ")
						: "");

		return sql;
	}

	class CountQuery implements Functor<String> {
		public Object apply(String input) {
			return "count(distinct " + input + ") as " + input;
		}
	}

	public List listDataModelElementValues(String onePathAmoebaQuery,
			String targetParameter) {
		try {
			AmoebaQueryParseResult query = new AmoebaQueryParseResult(
					onePathAmoebaQuery);

			ArrayList<String> conditionClause = new ArrayList<String>();
			for (String param : query.getParamList()) {
				if (param.equals(targetParameter))
					break;
				String condition = query.getConditionClause(param);
				if (condition != null)
					conditionClause.add(condition);
			}

			String sql = SQLExpression.fillTemplate("select $1 from $2 $3",
					targetParameter, "entry",
					(conditionClause.size() > 0) ? "where "
							+ StringUtil.join(conditionClause, " and ") : "");

			List<JSONValue> result = _query.selectColumnData(sql,
					targetParameter);
			return result;
		} catch (DBException e) {
			_logger.error(e);
			return new ArrayList<JSONValue>();
		}

	}

	public String getHTTPContent(String url) {
		try {
			BufferedReader in;
			if (!url.startsWith("http://")) {
				if (!url.startsWith("/"))
					url = "/" + url;
				_logger.debug("proxy request: " + url);
				in = WebApplicationResource.openResource(this
						.getServletContext(), url);
			} else {
				URL address = new URL(url);
				_logger.debug("proxy request: " + url);
				in = new BufferedReader(new InputStreamReader(address
						.openStream()));
			}

			StringWriter buffer = new StringWriter();
			BufferedWriter out = new BufferedWriter(buffer);
			String line;
			while ((line = in.readLine()) != null) {
				out.append(line);
				out.newLine();
			}
			in.close();
			out.flush();
			return buffer.toString();
		} catch (IOException e) {
			_logger.error(e);
			return "";
		}
	}

	public String getDatabaseCatalog(String jdbcAddress) {
		try {
			// TODO connection pool
			SQLiteAccess dbAccess = new SQLiteAccess(jdbcAddress);
			SQLiteCatalog catalog = dbAccess.getCatalog();
			String json = catalog.toJSON();
			_logger.debug("catalog: " + json);
			return json;
		} catch (DBException e) {
			_logger.error(e);
			return "";
		}
	}

	public String getTableData(String jdbcAddress, String tableName) {
		try {
			SQLiteAccess dbAccess = new SQLiteAccess(jdbcAddress);
			String sql = SQLExpression.fillTemplate("select * from $1",
					tableName);
			List<JSONObject> result = dbAccess.jsonQuery(sql);
			JSONObject resultObj = new JSONObject();
			JSONArray resultArray = new JSONArray();
			for (JSONObject row : result) {
				resultArray.add(row);
			}
			resultObj.put("data", resultArray);

			String json = resultObj.toJSONString();
			_logger.debug("query result: " + json);
			return json;
		} catch (DBException e) {
			_logger.error(e);
			return "";
		}
	}

	public List getTrackList(int entriesPerPage, int page) {
		ArrayList<TrackBean> trackList = new ArrayList<TrackBean>();
		ArrayList<String> trackXMLFileList = WebApplicationResource.find(this
				.getServletContext(), "/tracks/", "(.*)\\.xml$", true);
		for (String xmlPath : trackXMLFileList) {
			try {
				trackList.add(loadTrackInfo(xmlPath));
			} catch (UTGBException e) {
				_logger.error(e);
			}
		}

		return trackList;
	}

	private TrackBean loadTrackInfo(String trackXMLResourcePath)
			throws UTGBException {
		try {
			BufferedReader xmlReader = WebApplicationResource.openResource(this
					.getServletContext(), trackXMLResourcePath);
			TrackBean trackInfo = BeanUtil.createXMLBean(TrackBean.class,
					xmlReader);
			return trackInfo;
		} catch (XerialException e) {
			throw new UTGBException(e);
		} catch (IOException e) {
			throw new UTGBException(e);
		}
	}

	public List getTrackList(String prefix, int entriesPerPage, int page) {
		// TODO Auto-generated method stub
		return null;
	}

	public int numHitsOfTracks(String prefix) {
		// TODO Auto-generated method stub
		return 0;
	}

	public SearchResult keywordSearch(String keyword, int numEntriesPerPage,
			int page) {

		String keywordSearchURL = "http://alps3.gi.k.u-tokyo.ac.jp/~yamada/dev/keyword/search.php?query=" 
				+ keyword
				+ "&page="
				+ page
				+ "&pagewidth="
				+ numEntriesPerPage;
		_logger.debug(keywordSearchURL);

		boolean isScaffoldSearch = keyword.toLowerCase().contains("scaffold");
		
		try {
			URL url = new URL(keywordSearchURL);
			BufferedReader in = new BufferedReader(new InputStreamReader(url
					.openStream()));
			StringWriter buffer = new StringWriter();
			BufferedWriter out = new BufferedWriter(buffer);
			String line;
			while ((line = in.readLine()) != null) {
				out.append(line);
				out.newLine();
			}
			in.close();
			out.flush();
			String json = buffer.toString();
			_logger.debug(json);
			SearchResult searchResult = new SearchResult();
			BeanUtil.populateBean(searchResult, json);

			// parse the returned keywords
			String keywordLowerCase = keyword.toLowerCase();
			for(Iterator it = searchResult.getResult().iterator(); it.hasNext(); )
			{
				Result result = (Result) it.next();
				if(result.getSpecies() == null || result.getRevision() == null || result.getScaffold() == null)
					continue;
				
				ArrayList<Tag> matchedTagList = new ArrayList<Tag>();
				if(!isScaffoldSearch)
				{
					for(Iterator ti = result.getKeywords().iterator(); ti.hasNext(); )
					{
						Tag tag = (Tag) ti.next();
						if(tag.getKey().toLowerCase().contains(keywordLowerCase) || tag.getValue().toLowerCase().contains(keywordLowerCase))
						{
							matchedTagList.add(tag);
						}
					}
					
				}
				result.clearKeyword();
				for(Tag tag : matchedTagList)
				{
					result.addKeywords(tag);
				}
			}
			
			return searchResult;
		} catch (MalformedURLException e) {
			_logger.error(e);
		} catch (IOException e) {
			_logger.error(e);
		}
		catch (BeanException e) {
			_logger.error(e);
		}

		return null;
	}

	public void saveView(String viewXML) {
		HttpServletRequest request = this.getThreadLocalRequest();
		HttpSession session = request.getSession(true);
		session.setAttribute("view", viewXML);
	}
	
	public String getCurrentView()	{
		HttpServletRequest request = this.getThreadLocalRequest();
		HttpSession session = request.getSession(true);
		_logger.debug("session ID = " + session.getId());
		String viewXML = (String) session.getAttribute("view");
		if(viewXML == null || (viewXML != null && viewXML.length() == 0))
		{
			try {
				String xml = WebApplicationResource.getContent(this.getServletContext(), "/view/standard.xml");
				return xml;
			} catch (IOException e) {
				_logger.error(e);
				return null;
			}
		}
		return viewXML;
	}

}
