/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// Aqua Project
//
// TabViewer.java
// Since: 2007/03/24
//
// $Date: 2007-12-27 14:27:01 +0900 $
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/web/client/tab/TabViewer.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.web.client.tab;

import java.util.ArrayList;
import java.util.Iterator;

import org.utgenome.gwt.web.client.tab.TabEntry.TabEntryFactory;

import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.SourcesTabEvents;
import com.google.gwt.user.client.ui.TabBar;
import com.google.gwt.user.client.ui.TabListener;

/**
 * A TabViewer can have several TabEntry classes.
 * 
 * <code>
 * TabViewer v = new TabViewer();
 * v.addTabEntry(MyTab.factory());
 * v.addTabEntry(MyTab2.facory());
 * </code>
 * @author leo
 *
 */
public class TabViewer extends Composite implements TabListener {

	private ArrayList _tabEntryList = new ArrayList();
	private DockPanel _panel = new DockPanel();
	private TabBar _tabBar = new TabBar();
	private DockPanel _entryPanel = new DockPanel();
	
	public TabViewer()
	{
		_panel.add(_tabBar, DockPanel.NORTH);
		_tabBar.addTabListener(this);
		_panel.add(_entryPanel, DockPanel.CENTER);
		
		_panel.setWidth("100%");
		_entryPanel.setWidth("100%");
		
		initWidget(_panel);
	}
	
	public void selectTab(int index)
	{
		_tabBar.selectTab(index);
	}
	
	public void addTabEntry(final TabEntryFactory factory)
	{
		String name = factory.getTabName();
		_tabEntryList.add(factory);
		_tabBar.addTab(name);
	}

	public boolean onBeforeTabSelected(SourcesTabEvents sender, int tabIndex) {
		TabEntryFactory factory = (TabEntryFactory) _tabEntryList.get(tabIndex);
		
		// calling focus events
		for(Iterator it = _tabEntryList.iterator(); it.hasNext(); )
		{
			TabEntryFactory f = (TabEntryFactory) it.next();
			if(f.equals(factory))
				continue;	// skip the focused tab entry
			if(f.isInstanciated())
			{
				f.getInstance().onLostFocus();
			}
		}
		
		return true;
	}

	public void onTabSelected(SourcesTabEvents sender, int tabIndex) {
		_entryPanel.clear();
		TabEntryFactory factory = (TabEntryFactory) _tabEntryList.get(tabIndex);
		TabEntry entry = factory.getInstance();
		_entryPanel.add(entry, DockPanel.CENTER);
		entry.onFocus();
		
	}
}




