/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// UTGBMedaka Project
//
// ContextXMLGenerator.java
// Since: Aug 2, 2007
//
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/gwt/web/server/ContextXMLGenerator.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.gwt.web.server;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;

import org.xerial.util.FileResource;
import org.xerial.util.cui.OptionHandler;
import org.xerial.util.cui.OptionParser;
import org.xerial.util.cui.OptionParserException;
import org.xerial.util.template.Template;

public class ContextXMLGenerator {
	
	enum Opt { HELP, WEBAPP_NAME, APPLICATON_PATH, CONFIG_FILE, DOC_BASE };
	private static final OptionParser<Opt> opt = new OptionParser<Opt>();
	static 
	{
		opt.addOptionWithArgument(Opt.WEBAPP_NAME, "n", "name", "NAME", "web application name (e.g., UTGBMedka.war)");
		opt.addOptionWithArgument(Opt.APPLICATON_PATH, "p", "path", "APP_PATH", "web application path (e.g., /UTGBMedka)");
		opt.addOptionWithArgument(Opt.CONFIG_FILE, "c", "config", "CONFIG_FILE", "configuration file path");
		opt.addOptionWithArgument(Opt.DOC_BASE, "d", "docbase", "DOC_BASE", "document base path");

		opt.addOption(Opt.HELP, "h", "help", "display help message", new OptionHandler<Opt>() {
			public void handle(OptionParser<Opt> parser) throws OptionParserException {
				helpmessage();
			}
		});
	}
	
	private static void helpmessage()
	{
		System.out.println("usage: java -jar ContextXMLGenerator.jar -n webapp_name");
		System.out.println(opt.helpMessage());
	}
	
	public static void main(String args[]) throws IOException
	{
		try {
			opt.parse(args);
			
			if(!opt.isSet(Opt.WEBAPP_NAME))
				throw new OptionParserException("no webapp name is specified. use -s (webapp name)");
			
			Properties prop = new Properties();
			String webapp = opt.getValue(Opt.WEBAPP_NAME);
			prop.put("docbase", opt.isSet(Opt.DOC_BASE) ? opt.getValue(Opt.DOC_BASE) : webapp + ".war");
			prop.put("application_path", opt.isSet(Opt.APPLICATON_PATH) ? opt.getValue(Opt.APPLICATON_PATH) : "/" + webapp);
			prop.put("config_file", opt.isSet(Opt.CONFIG_FILE) ? opt.getValue(Opt.CONFIG_FILE) : webapp + ".config");
			System.out.println(generate(prop));
		} catch (OptionParserException e) {
			System.err.println(e);
		}
	}
	
	public static String generate(Properties property) throws IOException
	{
		URL templateURL = FileResource.find(ContextXMLGenerator.class, "context.xml.template");
		Template template = new Template(templateURL.openStream());
		return template.apply(property);
	}

}




