/*--------------------------------------------------------------------------
 *  Copyright 2007 utgenome.org
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *--------------------------------------------------------------------------*/
//--------------------------------------
// UTGB Common Project
//
// MethilIndex.java
// Since: Apr 27, 2007
//
// $Date: 2007-12-27 14:27:01 +0900 $
// $URL: http://svn.utgenome.org/utgb/trunk/utgb/utgb-core/src/main/java/org/utgenome/util/genome/MethilIndex.java $ 
// $Author: leo $
//--------------------------------------
package org.utgenome.util.genome;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;


/**
 * 
 * (id, name, compressed genome) 
 *  1   seq10   01001111011110000000
 *  2   seq11   0000001111000000000000000000000010
 * 
 * 
 * @author leo
 *
 */
public class MethilIndex 
{
	RandomAccessFile _file;
	
	public MethilIndex()
	{}

	
	public void compress(InputStream in) throws IOException
	{
		ByteArrayOutputStream buffer = new ByteArrayOutputStream();
		ZipOutputStream comressedStream = new ZipOutputStream(buffer);
		
		int r;
		while((r = in.read())!= -1)
		{
			comressedStream.write(r);	
		}
		
		
	}
	
	public void decompress(InputStream in) throws IOException
	{
		ZipInputStream compressedStream = new ZipInputStream(in);
		compressedStream.read();
	}
	
	
	public void changeMethylToBitString(InputStream in, OutputStream out) throws IOException
	{
		int r;
		
		byte byteString = 0;

		int cursor = 0;
		while((r = in.read()) != -1)
		{
			char c = (char) r; // TODO capitalize	
			char replacement;
			if(c == 'C' || c == 'c')
			{
				byteString |= (0x80 >> (cursor % 8));
				replacement = 'T';
			}
			else
				replacement = c;
			
			cursor++;
			if((cursor % 8) == 0)
			{
				out.write(byteString);
				byteString = 0;
			}
		}
		
		if((cursor % 8) > 0)
			out.write(byteString);
	}	

	
	public ArrayList<Integer> mythyledPosition(byte[] byteString)
	{
		ArrayList<Integer> result = new ArrayList<Integer>();
		for(int arrayIndex = 0; arrayIndex < byteString.length; arrayIndex++)
		{
			for(int i=0; i<8; i++)
			{
				if((byteString[arrayIndex] & (byte) (0x80 >> i)) > 0)
				{
					result.add(arrayIndex * 8 + i); 
				}
			}
		}
		return result;
	}
	
}




